﻿Imports System.Runtime.InteropServices
Imports UInt8 = System.Byte

Module modFelicaDump
    '==============
    ' モジュール名
    '==============
    Private Const mod_name As String = "modFelicaDump::dump()"

    '====================================
    '【関数名】dump
    '【引  数】なし
    '【戻り値】[out] String  FeliCaの情報
    '------------------------------------
    ' FeliCaの情報を取得する。
    '====================================
    Public Function dump() As String
        '----------
        ' 変数定義
        '----------
        Dim msg As New System.Text.StringBuilder
        Dim p_ptr As IntPtr
        Dim f_ptr, f_ptr2 As IntPtr
        Dim I, J, K As Integer
        Dim f, f2 As felica

        '--------
        ' 初期化
        '--------
        p_ptr = IntPtr.Zero
        f_ptr = IntPtr.Zero
        f_ptr2 = IntPtr.Zero

        Try
            '----------------------
            ' PaSoRi ハンドル取得
            '----------------------
            p_ptr = pasori_open(Nothing)
            If p_ptr = IntPtr.Zero Then
                MessageBox.Show("PaSoRi open failed.", mod_name, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return msg.ToString()
            End If

            '--------------------
            ' PaSoRi 初期化(接続)
            '--------------------
            pasori_init(p_ptr)

            '------------
            ' ポーリング
            '------------
            f_ptr = felica_polling(p_ptr, POLLING_ANY, 0, 0)
            If f_ptr = IntPtr.Zero Then
                MessageBox.Show("Polling card failed.", mod_name, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return msg.ToString()
            End If
            'felica構造体の実体を取得
            f = Marshal.PtrToStructure(f_ptr, GetType(felica))

            '-----------------
            ' IDm, PMm を取得
            '-----------------
            'IDm取得
            msg.Append("# IDm: " & hexdump(f.IDm, 8) & vbNewLine)
            'PMm取得
            msg.Append("# PMm: " & hexdump(f.PMm, 8) & vbNewLine)
            '改行
            msg.Append(vbNewLine)
            'ポインタ解放
            felica_free(f_ptr)

            '----------------------
            ' システムコードの列挙
            '----------------------
            f_ptr = felica_enum_systemcode(p_ptr)
            If f_ptr = IntPtr.Zero Then
                MessageBox.Show("Enum systemcode failed.", mod_name, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return msg.ToString()
            End If
            'felica構造体の実体を取得
            f = Marshal.PtrToStructure(f_ptr, GetType(felica))

            '------------------------------
            ' システムコード毎に情報を取得
            '------------------------------
            For I = 0 To f.num_system_code - 1
                '--------------------
                ' システムコード取得
                '--------------------
                msg.Append("# System code: " & N2HS(f.system_code(I)).ToString("X4") & vbNewLine)

                '------------------------------
                ' サービスコード毎に情報を取得
                '------------------------------
                f_ptr2 = felica_enum_service(p_ptr, N2HS(f.system_code(I)))
                If f_ptr2 = IntPtr.Zero Then
                    MessageBox.Show("Enum service failed.", mod_name, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    Return msg.ToString()
                End If
                'felica構造体の実体を取得
                f2 = Marshal.PtrToStructure(f_ptr2, GetType(felica))

                msg.Append("# Number of area = " & f2.num_area_code & vbNewLine)
                For J = 0 To f2.num_area_code - 1
                    '------------------
                    ' エリアコード取得
                    '------------------
                    msg.Append("# Area: " & f2.area_code(J).ToString("X4") & " - " & f2.end_service_code(J).ToString("X4") & vbNewLine)
                Next

                '--------------------
                ' サービスコード取得
                '--------------------
                msg.Append("# Number of service code = " & f2.num_service_code & vbNewLine)
                For J = 0 To f2.num_service_code - 1
                    Dim service As UInt16 = f2.service_code(J)
                    msg.Append(printserviceinfo(service))

                    '変数定義
                    Dim gch As GCHandle
                    Dim d_ptr As IntPtr

                    For K = 0 To 254
                        'データを格納する変数
                        Dim data(15) As UInt8

                        '変数dataのアドレスを取得
                        gch = GCHandle.Alloc(data, GCHandleType.Pinned)
                        d_ptr = gch.AddrOfPinnedObject()

                        '------------------------------------------------------------
                        ' サービスコード、ブロック番号を指定してブロックを読み込む
                        ' システムコードは felica_polling で指定したものが使用される
                        '------------------------------------------------------------
                        Dim err As Integer = felica_read_without_encryption02(f_ptr2, service, 0, CByte(K), d_ptr)
                        If err <> 0 Then
                            'エラーがあった場合は処理を抜ける
                            gch.Free() 'メモリ解放
                            Exit For
                        End If

                        '取得したデータを保持
                        msg.Append(service.ToString("X4") & ":" & K.ToString("X4") & " ")
                        msg.Append(hexdump(data, 16))
                        msg.Append(vbNewLine)

                        'メモリ解放
                        gch.Free()
                    Next
                Next

                '--------
                ' 後始末
                '--------
                msg.Append(vbNewLine)  '改行
                felica_free(f_ptr2)    'ポインタを解放
            Next

            '------------
            ' 接続を解除
            '------------
            felica_free(f_ptr)
            pasori_close(p_ptr)

            '------------
            ' 結果を返す
            '------------
            Return msg.ToString()
        Catch ex As Exception
            MessageBox.Show(ex.Message, mod_name, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return msg.ToString()
        End Try
    End Function

    '=================================
    '【関数名】printserviceinfo
    '【引  数】[in]  UInt16
    '【戻り値】[out] String
    '---------------------------------
    ' サービスコードの内容を取得する。
    '=================================
    Private Function printserviceinfo(ByVal s As UInt16) As String
        Dim ident As String
        Dim str As String

        Select Case ((s >> 1) And &HF)
            Case 0
                ident = "Area Code"
            Case 4
                ident = "Random Access R/W"
            Case 5
                ident = "Random Access Read only"
            Case 6
                ident = "Cyclic Access R/W"
            Case 7
                ident = "Cyclic Access Read only"
            Case 8
                ident = "Purse (Direct)"
            Case 9
                ident = "Purse (Cashback/decrement)"
            Case 10
                ident = "Purse (Decrement)"
            Case 11
                ident = "Purse (Read only)"
            Case Else
                ident = "INVALID or UNKOWN"
        End Select

        str = String.Format("# Serivce code = {0} : {1}", s.ToString("X4"), ident)
        If (s And &H1) = 0 Then
            str &= " (Protected)"
        End If
        str &= vbNewLine

        Return str
    End Function

    '=============================================
    '【関 数 名】hexdump
    '【第１引数】[in]  UInt8()  データ配列
    '【第２引数】[in]  Integer  配列のサイズ
    '【戻 り 値】[out] String   16進数文字列
    '---------------------------------------------
    ' 受け取ったデータを16進数の文字列に変換する。
    '=============================================
    Private Function hexdump(ByVal arg() As UInt8, ByVal size As Integer) As String
        Dim sbResult As New System.Text.StringBuilder

        For I As Integer = 0 To size - 1
            sbResult.Append(arg(I).ToString("X2") & " ")
        Next

        Return sbResult.ToString()
    End Function
End Module
